#! -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import pooler
from osv import fields
from osv import osv


#classe para a criação de grupos para as linhas de uma OV
class sale_order_line_groups(osv.osv):
    _name = "sale.order.line_groups"

    _columns = {
        'name'      : fields.char('Nome do grupo', size=100),
        'order_id'  : fields.many2one('sale.order', 'Sale order', invisible=True),
        'sequence'  : fields.float('Sequência'),
        'subtotal'  : fields.float('Subtotal'),
    }

    _defaults = {
        'subtotal': 0,
    }

sale_order_line_groups()




class sale_order_line(osv.osv):
    _inherit = "sale.order.line"

    _columns = {
        #altera-se para float para permitir um melhor controlo dos valores
        'sequence': fields.float('Sequence', help="Gives the sequence order when displaying a list of sales order lines."),

        'line_group': fields.many2one('sale.order.line_groups', 'Group'),
    }


    #isto devia de ser feito quando se coloca as linhas e não tanto na sua gravação

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}

        aux=super(sale_order_line, self).create(cr, uid, vals, context=context)

        objLin  = self.pool.get('sale.order.line')
        objGr   = self.pool.get('sale.order.line_groups')

        order_id    = vals.get('order_id')
        line_group  = vals.get('line_group')
        product_id  = vals.get('product_id')

        if line_group and product_id:
            grupo=objGr.read(cr, uid, line_group, ['id','name'])

            #se ainda não existir uma linha para o grupo, cria-se a dita
            res=objLin.search(cr, uid, [
                                        ('product_id','=',False),
                                        ('line_group','=',line_group)
                                        ])
            if not res:
                objLin.create(cr, uid, {
                                        'order_id'          : order_id,
                                        'line_group'        : line_group,
                                        'name'              : grupo['name'],
                                        'product_uos_qty'   : 0,
                                        'sequence'          : 999999
                                        })
                #cria-se a associação do grupo à OV
                objGr.write(cr, uid, grupo['id'], {'order_id': order_id})

        #se não tiver sido indicado um grupo, tenta-se criar um "Diversos"
        if not line_group:
            res=objGr.search(cr, uid, [('order_id','=',order_id),('name','=','Diversos')])
            if not res:
                res=objGr.create(cr, uid, {
                                            'name'      : 'Diversos',
                                            'order_id'  : order_id,
                                            'sequence'  : 999999
                                            })

                objLin.create(cr, uid, {
                                        'order_id'          : order_id,
                                        'line_group'        : res,
                                        'name'              : 'Diversos',
                                        'product_uos_qty'   : 0
                                        })
            else:
                res=res[0]

            cr.execute("UPDATE sale_order_line SET line_group=%s WHERE id=%s" % (res,aux))

        self.pool.get('sale.order').ordenar_linhas(cr, uid, order_id)

        return aux


    def write(self, cr, uid, ids, vals, context=None):
        # Update parent and siblings or children records
        if isinstance(ids, (int, long)):
            ids = [ids]

        for ov in self.browse(cr, uid, ids, context=context):
            #is podia estar melhor já que a ordenação só deveria acontecer se tivesse sido alterada alguma inha...
            self.pool.get('sale.order').ordenar_linhas(cr, uid, ov.id)

        return super(sale_order_line,self).write(cr, uid, ids, vals, context=context)



sale_order_line()



class sale_order(osv.osv):
    _inherit = "sale.order"


    def ordenar_linhas(self, cr, uid, order_id, context=None):
        """"
        ordena-se as linhas
        1) linhas sem grupo
        2) linhas com grupo
        """
        txtSQL="SELECT  A.id, \
                        CASE WHEN COALESCE(A.product_id,0)=0 THEN 999999 ELSE A.sequence END AS sequence, \
                        COALESCE(A.product_id,0) AS product_id \
                FROM    sale_order_line A \
                        INNER JOIN sale_order B ON A.order_id=B.id \
                        INNER JOIN sale_order_line_groups C ON B.id=C.order_id AND A.line_group=C.id \
                WHERE   A.order_id=%s \
                ORDER   BY \
                        C.sequence DESC, \
                        A.line_group, \
                        CASE WHEN COALESCE(A.product_id,0)=0 THEN 999999 ELSE A.sequence END DESC, \
                        A.product_id ASC" % (order_id)
        cr.execute(txtSQL)
        res = cr.dictfetchall()

        objLin=self.pool.get('sale.order.line')
        contador=0
        for r in res:
            contador=contador+1
            vals={'sequence':contador}
            if not r['product_id']:
                vals['product_uos_qty']=0

            objLin.write(cr, uid, r['id'], vals)

        return True



    #cálculo de subtotais por grupo
    def calcular_subtotal(self, cr, uid, order_id, context=None):
        objLin=self.pool.get('sale.order.line')

        #quais os grupos associados à OV?
        txtSQL="SELECT DISTINCT(line_group) as line_group FROM sale_order_line WHERE order_id=%s" % (order_id)
        cr.execute(txtSQL)
        res = cr.dictfetchall()
        for r in res:
            res2=objLin.search(cr, uid, [('order_id','=',order_id),('line_group','=',r['line_group'])])
            res2=objLin.browse(cr, uid, res2)
            subtotal=0
            for r2 in res2:
                subtotal=subtotal+r2['price_subtotal']

            self.pool.get('sale.order.line_groups').write(cr, uid, r['line_group'], {'subtotal':subtotal})

        return 0



    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}

        order_id=super(sale_order, self).create(cr, uid, vals, context=context)

        self.ordenar_linhas(cr, uid, order_id)
        self.calcular_subtotal(cr, uid, order_id)

        return order_id



    def write(self, cr, uid, ids, vals, context=None):
        # Update parent and siblings or children records
        if isinstance(ids, (int, long)):
            ids = [ids]

        aux=super(sale_order,self).write(cr, uid, ids, vals, context=context)

        for ov in self.browse(cr, uid, ids, context=context):
            #isto podia estar melhor já que a ordenação só deveria acontecer se tivesse sido alterada alguma linha...
            self.ordenar_linhas(cr, uid, ov.id)
            self.calcular_subtotal(cr, uid, ov.id)

        return aux



sale_order()